@@ -15,7 +15,6 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-#include <boost/python.hpp>
 #include "graph.hh"
 #include "graph_filtering.hh"
 #include "graph_util.hh"
@@ -28,6 +27,8 @@
 
 #include "random.hh"
 
+#include <boost/python.hpp>
+
 using namespace std;
 using namespace boost;
 using namespace graph_tool;
@@ -121,7 +122,7 @@ struct add_ptr
 };
 
 template <class State>
-void export_state()
+static void export_state()
 {
     mpl::for_each<all_graph_views, add_ptr>
         ([](auto g)
@@ -132,7 +133,7 @@ void export_state()
 }
 
 template <template <bool...> class Base, bool... As>
-void export_SI_state()
+static void export_SI_state()
 {
     export_state<Base<As..., false, false>>();
     export_state<Base<As..., true, false>>();
