@@ -31,7 +31,7 @@ using namespace boost;
 // we need a min() function with arguments of different types
 
 template <class T1, class T2>
-typename boost::mpl::if_<
+static typename boost::mpl::if_<
     typename boost::mpl::or_<typename std::is_floating_point<T1>::type,
                              typename std::is_floating_point<T2>::type>::type,
     double, int>::type
@@ -48,13 +48,8 @@ min(const T1& v1, const T2& v2)
 using namespace graph_tool;
 using namespace boost;
 
-#if (BOOST_VERSION >= 104400)
-# include <boost/graph/boykov_kolmogorov_max_flow.hpp>
-# define KOLMOGOROV_MAX_FLOW boost::boykov_kolmogorov_max_flow
-#else
-# include <boost/graph/kolmogorov_max_flow.hpp>
-# define KOLMOGOROV_MAX_FLOW boost::kolmogorov_max_flow
-#endif
+#include <boost/graph/boykov_kolmogorov_max_flow.hpp>
+#define KOLMOGOROV_MAX_FLOW boost::boykov_kolmogorov_max_flow
 
 struct get_kolmogorov_max_flow
 {
