@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2020 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2021 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -21,7 +21,6 @@
 
 #include <boost/lambda/lambda.hpp>
 #include <boost/lambda/bind.hpp>
-#include <iostream>
 
 using namespace std;
 using namespace boost;
@@ -96,7 +95,13 @@ struct clear_vertices
 
 void GraphInterface::clear()
 {
-    run_action<>()(*this, std::bind(clear_vertices(), std::placeholders::_1))();
+    run_action<>()
+        (*this,
+         [&](auto&& graph)
+         {
+             return clear_vertices()
+                 (std::forward<decltype(graph)>(graph));
+         })();
 }
 
 struct do_clear_edges
@@ -111,5 +116,11 @@ struct do_clear_edges
 
 void GraphInterface::clear_edges()
 {
-    run_action<>()(*this, std::bind(do_clear_edges(), std::placeholders::_1))();
+    run_action<>()
+        (*this,
+         [&](auto&& graph)
+         {
+             return do_clear_edges()
+                 (std::forward<decltype(graph)>(graph));
+         })();
 }
