@@ -15,6 +15,8 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 
+#include "graph.hh"
+
 #include <boost/python/extract.hpp>
 
 #include <iostream>
@@ -42,11 +44,7 @@ using namespace boost;
 using namespace graph_tool;
 
 // use correct smart pointer type for dynamic properties
-#if (BOOST_VERSION / 100 % 1000 >= 44)
-    #define DP_SMART_PTR boost::shared_ptr
-#else
-    #define DP_SMART_PTR std::auto_ptr
-#endif
+#define DP_SMART_PTR boost::shared_ptr
 
 //
 // Persistent IO of python::object types. All the magic is done in python,
@@ -80,7 +78,7 @@ python::object lexical_cast<python::object,string>(const string& ps)
 
 namespace python
 {
-std::ostringstream& operator<<(std::ostringstream& stream, const boost::python::object& o)
+static std::ostringstream& operator<<(std::ostringstream& stream, const boost::python::object& o)
 {
     stream << base64_encode(lexical_cast<string>(o));
     return stream;
