@@ -15,8 +15,8 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-#include "graph_python_interface.hh"
 #include "graph.hh"
+#include "graph_python_interface.hh"
 #include "graph_properties.hh"
 #include "graph_filtering.hh"
 #include "graph_selectors.hh"
@@ -231,7 +231,7 @@ void infect_vertex_property(GraphInterface& gi, boost::any prop,
 }
 
 template <class Value>
-vector<Value> operator-(const vector<Value>& a, const vector<Value>& b)
+static vector<Value> operator-(const vector<Value>& a, const vector<Value>& b)
 {
     vector<Value> c(a);
     c.resize(max(a.size(), b.size()), Value(0));
