@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2020 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2021 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -21,8 +21,6 @@
 #include "demangle.hh"
 
 #include <boost/python.hpp>
-#include <boost/lambda/bind.hpp>
-#include <functional>
 
 using namespace std;
 using namespace boost;
@@ -77,9 +75,13 @@ struct export_vertex_property_map
                                       boost::mpl::quote1<std::add_pointer> >::type all_const_graph_views;
         typedef boost::mpl::joint_view<all_graph_views, all_const_graph_views>::type graph_views;
 
-        boost::mpl::for_each<graph_views>(std::bind(dispatch_access<PropertyMap>(),
-                                                    std::placeholders::_1,
-                                                    std::ref(pclass), return_policy()));
+        boost::mpl::for_each<graph_views>(
+            [&](auto&& graph)
+            {
+                return dispatch_access<PropertyMap>()(
+                    std::forward<decltype(graph)>(graph), pclass,
+                    return_policy());
+            });
     }
 
     template <class PropertyMap>
@@ -162,9 +164,13 @@ struct export_edge_property_map
                                       boost::mpl::quote1<std::add_pointer> >::type all_const_graph_views;
         typedef boost::mpl::joint_view<all_graph_views, all_const_graph_views>::type graph_views;
 
-        boost::mpl::for_each<graph_views>(std::bind(dispatch_access<PropertyMap>(),
-                                                    std::placeholders::_1,
-                                                    std::ref(pclass), return_policy()));
+        boost::mpl::for_each<graph_views>(
+            [&](auto&& graph)
+            {
+                return dispatch_access<PropertyMap>()(
+                    std::forward<decltype(graph)>(graph), pclass,
+                    return_policy());
+            });
     }
 };
 
