@@ -59,8 +59,6 @@ void export_sbm_state()
              double (state_t::*get_move_prob)(size_t, size_t, size_t, double,
                                               double, bool) =
                  &state_t::get_move_prob;
-             void (state_t::*merge_vertices)(size_t, size_t) =
-                 &state_t::merge_vertices;
              void (state_t::*set_partition)(boost::any&) =
                  &state_t::set_partition;
 
@@ -73,19 +71,16 @@ void export_sbm_state()
                  .def("move_vertices", move_vertices)
                  .def("set_partition", set_partition)
                  .def("virtual_move", virtual_move)
-                 .def("merge_vertices", merge_vertices)
                  .def("sample_block", sample_block)
                  .def("sample_neighbor", random_neighbor)
                  .def("entropy", &state_t::entropy)
                  .def("get_partition_dl", &state_t::get_partition_dl)
                  .def("get_deg_dl", &state_t::get_deg_dl)
                  .def("get_move_prob", get_move_prob)
-                 .def("enable_partition_stats",
-                      &state_t::enable_partition_stats)
-                 .def("disable_partition_stats",
-                      &state_t::disable_partition_stats)
-                 .def("is_partition_stats_enabled",
-                      &state_t::is_partition_stats_enabled)
+                 .def("reset_partition_stats",
+                      &state_t::reset_partition_stats)
+                 .def("init_partition_stats",
+                      &state_t::init_partition_stats)
                  .def("couple_state",
                       &state_t::couple_state)
                  .def("decouple_state",
@@ -96,8 +91,6 @@ void export_sbm_state()
                       &state_t::get_B_E_D)
                  .def("clear_egroups",
                       &state_t::clear_egroups)
-                 .def("rebuild_neighbor_sampler",
-                      &state_t::rebuild_neighbor_sampler)
                  .def("sync_emat",
                       &state_t::sync_emat)
                  .def("get_edge_sampler",
