@@ -42,7 +42,7 @@ void vector_map(boost::python::object ovals, boost::python::object omap)
 }
 
 template <class Value>
-void vector_continuous_map(boost::python::object ovals)
+void vector_contiguous_map(boost::python::object ovals)
 {
     multi_array_ref<Value,1> vals = get_array<Value,1>(ovals);
     gt_hash_map<Value, size_t> map;
@@ -74,7 +74,7 @@ extern void export_blockmodel_mcmc();
 extern void export_blockmodel_multicanonical();
 extern void export_blockmodel_multicanonical_multiflip();
 extern void export_blockmodel_multiflip_mcmc();
-extern void export_blockmodel_merge();
+extern void export_blockmodel_multilevel_mcmc();
 extern void export_blockmodel_gibbs();
 extern void export_overlap_blockmodel_state();
 extern void export_overlap_blockmodel_mcmc();
@@ -82,15 +82,16 @@ extern void export_overlap_blockmodel_mcmc_bundled();
 extern void export_overlap_blockmodel_multicanonical();
 extern void export_overlap_blockmodel_multicanonical_multiflip();
 extern void export_overlap_blockmodel_multiflip_mcmc();
+extern void export_overlap_blockmodel_multilevel_mcmc();
 extern void export_overlap_blockmodel_gibbs();
 extern void export_overlap_blockmodel_vacate();
 extern void export_layered_blockmodel_state();
 extern void export_layered_blockmodel_mcmc();
-extern void export_layered_blockmodel_merge();
 extern void export_layered_blockmodel_gibbs();
 extern void export_layered_blockmodel_multicanonical();
 extern void export_layered_blockmodel_multicanonical_multiflip();
 extern void export_layered_blockmodel_multiflip_mcmc();
+extern void export_layered_blockmodel_multilevel_mcmc();
 extern void export_layered_overlap_blockmodel_state();
 extern void export_layered_overlap_blockmodel_mcmc();
 extern void export_layered_overlap_blockmodel_bundled_mcmc();
@@ -98,6 +99,7 @@ extern void export_layered_overlap_blockmodel_gibbs();
 extern void export_layered_overlap_blockmodel_multicanonical();
 extern void export_layered_overlap_blockmodel_multicanonical_multiflip();
 extern void export_layered_overlap_blockmodel_multiflip_mcmc();
+extern void export_layered_overlap_blockmodel_multilevel_mcmc();
 extern void export_layered_overlap_blockmodel_vacate();
 extern void export_em_blockmodel_state();
 extern void export_blockmodel_exhaustive();
@@ -127,21 +129,31 @@ extern void export_pseudo_ising_mcmc_h();
 extern void export_vi_center_state();
 extern void export_vi_center_mcmc();
 extern void export_vi_multiflip_mcmc();
+extern void export_vi_multilevel_mcmc();
 extern void export_rmi_center_state();
 extern void export_rmi_center_mcmc();
 extern void export_rmi_multiflip_mcmc();
+extern void export_rmi_multilevel_mcmc();
 extern void export_partition_mode();
 extern void export_mode_cluster_state();
 extern void export_mode_cluster_mcmc();
 extern void export_mode_cluster_multiflip_mcmc();
+extern void export_mode_cluster_multilevel_mcmc();
+extern void export_pp_gibbs();
 extern void export_pp_state();
 extern void export_pp_mcmc();
 extern void export_pp_multiflip_mcmc();
+extern void export_pp_multilevel_mcmc();
 extern void export_modularity_state();
+extern void export_modularity_gibbs();
 extern void export_modularity_mcmc();
 extern void export_modularity_multiflip_mcmc();
+extern void export_modularity_multilevel_mcmc();
 extern void export_latent_closure_state();
 extern void export_latent_closure_mcmc();
+extern void export_hist_state();
+extern void export_hist_mcmc();
+extern void export_clique_decomposition();
 
 BOOST_PYTHON_MODULE(libgraph_tool_inference)
 {
@@ -152,7 +164,7 @@ BOOST_PYTHON_MODULE(libgraph_tool_inference)
     export_blockmodel_multicanonical();
     export_blockmodel_multicanonical_multiflip();
     export_blockmodel_multiflip_mcmc();
-    export_blockmodel_merge();
+    export_blockmodel_multilevel_mcmc();
     export_blockmodel_gibbs();
     export_overlap_blockmodel_state();
     export_overlap_blockmodel_mcmc();
@@ -160,12 +172,13 @@ BOOST_PYTHON_MODULE(libgraph_tool_inference)
     export_overlap_blockmodel_multicanonical();
     export_overlap_blockmodel_multicanonical_multiflip();
     export_overlap_blockmodel_multiflip_mcmc();
+    export_overlap_blockmodel_multilevel_mcmc();
     export_overlap_blockmodel_gibbs();
     export_overlap_blockmodel_vacate();
     export_layered_blockmodel_state();
     export_layered_blockmodel_mcmc();
     export_layered_blockmodel_multiflip_mcmc();
-    export_layered_blockmodel_merge();
+    export_layered_blockmodel_multilevel_mcmc();
     export_layered_blockmodel_gibbs();
     export_layered_blockmodel_multicanonical();
     export_layered_blockmodel_multicanonical_multiflip();
@@ -176,6 +189,7 @@ BOOST_PYTHON_MODULE(libgraph_tool_inference)
     export_layered_overlap_blockmodel_multicanonical();
     export_layered_overlap_blockmodel_multicanonical_multiflip();
     export_layered_overlap_blockmodel_multiflip_mcmc();
+    export_layered_overlap_blockmodel_multilevel_mcmc();
     export_layered_overlap_blockmodel_vacate();
     export_em_blockmodel_state();
     export_blockmodel_exhaustive();
@@ -205,26 +219,36 @@ BOOST_PYTHON_MODULE(libgraph_tool_inference)
     export_vi_center_state();
     export_vi_center_mcmc();
     export_vi_multiflip_mcmc();
+    export_vi_multilevel_mcmc();
     export_rmi_center_state();
     export_rmi_center_mcmc();
     export_rmi_multiflip_mcmc();
+    export_rmi_multilevel_mcmc();
     export_partition_mode();
     export_mode_cluster_state();
     export_mode_cluster_mcmc();
     export_mode_cluster_multiflip_mcmc();
+    export_mode_cluster_multilevel_mcmc();
     export_pp_state();
+    export_pp_gibbs();
     export_pp_mcmc();
     export_pp_multiflip_mcmc();
+    export_pp_multilevel_mcmc();
     export_modularity_state();
+    export_modularity_gibbs();
     export_modularity_mcmc();
     export_modularity_multiflip_mcmc();
+    export_modularity_multilevel_mcmc();
     export_latent_closure_state();
     export_latent_closure_mcmc();
+    export_hist_state();
+    export_hist_mcmc();
+    export_clique_decomposition();
 
     def("vector_map", vector_map<int32_t>);
     def("vector_map64", vector_map<int64_t>);
     def("vector_rmap", vector_rmap<int32_t>);
     def("vector_rmap64", vector_rmap<int64_t>);
-    def("vector_continuous_map", vector_continuous_map<int32_t>);
-    def("vector_continuous_map64", vector_continuous_map<int64_t>);
+    def("vector_contiguous_map", vector_contiguous_map<int32_t>);
+    def("vector_contiguous_map64", vector_contiguous_map<int64_t>);
 }
