@@ -15,11 +15,12 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-#include <boost/python.hpp>
-
+#include "graph.hh"
 #include "graph_tool.hh"
 #include "random.hh"
 
+#include <boost/python.hpp>
+
 #define GRAPH_BLOCKMODEL_RMAP_ENABLE
 #include "../blockmodel/graph_blockmodel_util.hh"
 #include "../blockmodel/graph_blockmodel.hh"
@@ -62,8 +63,6 @@ void export_lsbm()
                       double (state_t::*get_move_prob)(size_t, size_t, size_t, double,
                                                        double, bool)
                           = &state_t::get_move_prob;
-                      void (state_t::*merge_vertices)(size_t, size_t)
-                          = &state_t::merge_vertices;
                       void (state_t::*set_partition)(boost::any&)
                           = &state_t::set_partition;
                       void (state_t::*move_vertices)(python::object, python::object) =
@@ -87,7 +86,6 @@ void export_lsbm()
                           .def("move_vertices", move_vertices)
                           .def("set_partition", set_partition)
                           .def("virtual_move", virtual_move)
-                          .def("merge_vertices", merge_vertices)
                           .def("sample_block", sample_block)
                           .def("entropy", &state_t::entropy)
                           .def("get_partition_dl", &state_t::get_partition_dl)
@@ -105,16 +103,12 @@ void export_lsbm()
                                 {
                                     return python::object(block_state_t(state.get_layer(l)));
                                 })
-                          .def("enable_partition_stats",
-                               &state_t::enable_partition_stats)
-                          .def("disable_partition_stats",
-                               &state_t::disable_partition_stats)
-                          .def("is_partition_stats_enabled",
-                               &state_t::is_partition_stats_enabled)
+                          .def("reset_partition_stats",
+                               &state_t::reset_partition_stats)
+                          .def("init_partition_stats",
+                               &state_t::init_partition_stats)
                           .def("clear_egroups",
                                &state_t::clear_egroups)
-                          .def("rebuild_neighbor_sampler",
-                               &state_t::rebuild_neighbor_sampler)
                           .def("sync_emat",
                                &state_t::sync_emat)
                           .def("sync_bclabel",
