@@ -115,16 +115,12 @@ void export_layered_overlap_blockmodel_state()
                                 {
                                     return python::object(block_state_t(state.get_layer(l)));
                                 })
-                          .def("enable_partition_stats",
-                               &state_t::enable_partition_stats)
-                          .def("disable_partition_stats",
-                               &state_t::disable_partition_stats)
-                          .def("is_partition_stats_enabled",
-                               &state_t::is_partition_stats_enabled)
+                          .def("reset_partition_stats",
+                               &state_t::reset_partition_stats)
+                          .def("init_partition_stats",
+                               &state_t::init_partition_stats)
                           .def("clear_egroups",
                                &state_t::clear_egroups)
-                          .def("rebuild_neighbor_sampler",
-                               &state_t::rebuild_neighbor_sampler)
                           .def("sync_emat",
                                &state_t::sync_emat)
                           .def("sync_bclabel",
