@@ -31,16 +31,32 @@ constexpr size_t overlap_stats_t::_null;
 
 GEN_DISPATCH(overlap_block_state, OverlapBlockState, OVERLAP_BLOCK_STATE_params)
 
-python::object make_overlap_block_state(boost::python::object ostate,
-                                        rng_t& rng)
+python::object make_overlap_block_state(boost::python::object ostate)
 {
     python::object state;
     overlap_block_state::make_dispatch(ostate,
-                                       [&](auto& s){state = python::object(s);},
-                                       rng);
+                                       [&](auto& s){state = python::object(s);});
     return state;
 }
 
+template <class Edge, class Graph>
+inline typename graph_traits<Graph>::vertex_descriptor
+get_source(const Edge& e, const Graph &g)
+{
+    if constexpr (is_directed_::apply<Graph>::type::value)
+        return source(e, g);
+    return std::min(source(e, g), target(e, g));
+}
+
+template <class Edge, class Graph>
+inline typename graph_traits<Graph>::vertex_descriptor
+get_target(const Edge& e, const Graph &g)
+{
+    if constexpr (is_directed_::apply<Graph>::type::value)
+        return target(e, g);
+    return std::max(source(e, g), target(e, g));
+}
+
 void get_eg_overlap(GraphInterface& gi, GraphInterface& egi, boost::any obe,
                     boost::any ob, boost::any onode_index,
                     boost::any ohalf_edges, boost::any oeindex, boost::any orec,
@@ -187,20 +203,16 @@ void export_overlap_blockmodel_state()
                       &state_t::get_B_E)
                  .def("get_B_E_D",
                       &state_t::get_B_E_D)
-                 .def("enable_partition_stats",
-                      &state_t::enable_partition_stats)
-                 .def("disable_partition_stats",
-                      &state_t::disable_partition_stats)
-                 .def("is_partition_stats_enabled",
-                      &state_t::is_partition_stats_enabled)
+                 .def("init_partition_stats",
+                      &state_t::init_partition_stats)
+                 .def("reset_partition_stats",
+                      &state_t::reset_partition_stats)
                  .def("couple_state",
                       &state_t::couple_state)
                  .def("decouple_state",
                       &state_t::decouple_state)
                  .def("clear_egroups",
                       &state_t::clear_egroups)
-                 .def("rebuild_neighbor_sampler",
-                      &state_t::rebuild_neighbor_sampler)
                  .def("sync_emat",
                       &state_t::sync_emat)
                  .def("get_be_overlap",
