@@ -15,13 +15,15 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-#include <boost/graph/hawick_circuits.hpp>
+#include "graph.hh"
 
 #include "graph_tool.hh"
 #include "numpy_bind.hh"
 #include "coroutine.hh"
 #include "graph_python_interface.hh"
 
+#include "boost/graph/hawick_circuits.hpp"
+
 using namespace std;
 using namespace graph_tool;
 
