@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2020 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2021 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -42,7 +42,12 @@ typedef property_map_types::apply<mpl::vector<int32_t>,
 void dominator_tree(GraphInterface& gi, size_t entry, boost::any pred_map)
 {
     run_action<graph_tool::detail::always_directed>()
-        (gi, std::bind(get_dominator_tree(), std::placeholders::_1, entry,
-                       std::placeholders::_2),
+        (gi,
+         [&](auto&& graph, auto&& a2)
+         {
+             return get_dominator_tree()
+                 (std::forward<decltype(graph)>(graph), entry,
+                  std::forward<decltype(a2)>(a2));
+         },
          pred_properties())(pred_map);
 }
