@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2020 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2021 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -120,9 +120,16 @@ void get_kruskal_spanning_tree(GraphInterface& gi, boost::any weight_map,
         weight_maps;
 
     run_action<graph_tool::detail::never_directed>()
-        (gi, std::bind(get_kruskal_min_span_tree(), std::placeholders::_1, gi.get_vertex_index(),
-                       std::placeholders::_2, std::placeholders::_3),
-         weight_maps(), writable_edge_scalar_properties())(weight_map, tree_map);
+        (gi,
+         [&](auto&& graph, auto&& a2, auto&& a3)
+         {
+             return get_kruskal_min_span_tree()
+                 (std::forward<decltype(graph)>(graph), gi.get_vertex_index(),
+                  std::forward<decltype(a2)>(a2),
+                  std::forward<decltype(a3)>(a3));
+         },
+         weight_maps(),
+         writable_edge_scalar_properties())(weight_map, tree_map);
 }
 
 void get_prim_spanning_tree(GraphInterface& gi, size_t root,
@@ -137,7 +144,13 @@ void get_prim_spanning_tree(GraphInterface& gi, size_t root,
         weight_maps;
 
     run_action<graph_tool::detail::never_directed>()
-        (gi, std::bind(get_prim_min_span_tree(), std::placeholders::_1, root,
-                       gi.get_vertex_index(), std::placeholders::_2, std::placeholders::_3),
+        (gi,
+         [&](auto&& graph, auto&& a2, auto&& a3)
+         {
+             return get_prim_min_span_tree()
+                 (std::forward<decltype(graph)>(graph), root,
+                  gi.get_vertex_index(), std::forward<decltype(a2)>(a2),
+                  std::forward<decltype(a3)>(a3));
+         },
          weight_maps(), tree_properties())(weight_map, tree_map);
 }
