@@ -1,6 +1,6 @@
 // graph-tool -- a general graph modification and manipulation thingy
 //
-// Copyright (C) 2006-2020 Tiago de Paula Peixoto <tiago@skewed.de>
+// Copyright (C) 2006-2021 Tiago de Paula Peixoto <tiago@skewed.de>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU Lesser General Public License as published by the Free
@@ -40,8 +40,12 @@ bool topological_sort(GraphInterface& gi, vector<int32_t>& sort)
     try
     {
         run_action<>()
-            (gi, std::bind(get_topological_sort(), std::placeholders::_1,
-                           std::ref(sort)))();
+            (gi,
+             [&](auto&& graph)
+             {
+                 return get_topological_sort()
+                     (std::forward<decltype(graph)>(graph), sort);
+             })();
         return true;
     }
     catch (not_a_dag& e)
