@@ -55,6 +55,19 @@ State classes
    ~graph_tool.inference.planted_partition.PPBlockState
    ~graph_tool.inference.modularity.ModularityState
    ~graph_tool.inference.mcmc.TemperingState
+   ~graph_tool.inference.clique_decomposition.CliqueState
+
+Abstract base classes
+=====================
+.. autosummary::
+   :nosignatures:
+
+   ~graph_tool.inference.base_states.MCMCState
+   ~graph_tool.inference.base_states.MultiflipMCMCState
+   ~graph_tool.inference.base_states.MultilevelMCMCState
+   ~graph_tool.inference.base_states.GibbsMCMCState
+   ~graph_tool.inference.base_states.MulticanonicalMCMCState
+   ~graph_tool.inference.base_states.ExhaustiveSweepState
 
 Sampling and minimization
 =========================
@@ -64,11 +77,8 @@ Sampling and minimization
 
    ~graph_tool.inference.mcmc.mcmc_equilibrate
    ~graph_tool.inference.mcmc.mcmc_anneal
-   ~graph_tool.inference.mcmc.mcmc_multilevel
    ~graph_tool.inference.mcmc.multicanonical_equilibrate
    ~graph_tool.inference.mcmc.MulticanonicalState
-   ~graph_tool.inference.bisection.bisection_minimize
-   ~graph_tool.inference.nested_blockmodel.hierarchy_minimize
 
 Comparing and manipulating partitions
 =====================================
@@ -93,6 +103,8 @@ Comparing and manipulating partitions
    ~graph_tool.inference.partition_modes.partition_overlap_center
    ~graph_tool.inference.partition_modes.nested_partition_overlap_center
    ~graph_tool.inference.partition_modes.nested_partition_clear_null
+   ~graph_tool.inference.util.contiguous_map
+   ~graph_tool.inference.util.nested_contiguous_map
 
 Auxiliary functions
 ===================
@@ -100,7 +112,6 @@ Auxiliary functions
 .. autosummary::
    :nosignatures:
 
-   ~graph_tool.inference.blockmodel.model_entropy
    ~graph_tool.inference.blockmodel.mf_entropy
    ~graph_tool.inference.blockmodel.bethe_entropy
    ~graph_tool.inference.blockmodel.microstate_entropy
@@ -205,17 +216,21 @@ __all__ = ["minimize_blockmodel_dl",
            "LatentLayerBaseState",
            "LatentClosureBlockState",
            "MeasuredClosureBlockState",
+           "HistState",
+           "CliqueState",
+           "MCMCState",
+           "MultiflipMCMCState",
+           "MultilevelMCMCState",
+           "GibbsMCMCState",
+           "MulticanonicalMCMCState",
+           "ExhaustiveSweepState",
            "mcmc_equilibrate",
            "mcmc_anneal",
-           "mcmc_multilevel",
            "TemperingState",
            "multicanonical_equilibrate",
            "MulticanonicalState",
-           "bisection_minimize",
-           "hierarchy_minimize",
            "EMBlockState",
            "em_infer",
-           "model_entropy",
            "mf_entropy",
            "bethe_entropy",
            "microstate_entropy",
@@ -234,6 +249,8 @@ __all__ = ["minimize_blockmodel_dl",
            "partition_overlap",
            "nested_partition_overlap",
            "contingency_graph",
+           "contiguous_map",
+           "nested_contiguous_map",
            "align_partition_labels",
            "align_nested_partition_labels",
            "shuffle_nested_partition_labels",
@@ -247,13 +264,13 @@ __all__ = ["minimize_blockmodel_dl",
            "mutual_information",
            "reduced_mutual_information"]
 
+from . base_states import *
 from . blockmodel import *
 from . overlap_blockmodel import *
 from . layered_blockmodel import *
 from . nested_blockmodel import *
 from . uncertain_blockmodel import *
 from . mcmc import *
-from . bisection import *
 from . minimize import *
 from . blockmodel_em import *
 from . util import *
@@ -263,3 +280,5 @@ from . partition_centroid import *
 from . partition_modes import *
 from . planted_partition import *
 from . latent_layers import *
+from . histogram import *
+from . clique_decomposition import *
