@@ -270,118 +270,6 @@ def mcmc_anneal(state, beta_range=(1., 10.), niter=100, history=False,
     else:
         return S, nattempts, nmoves
 
-
-def mcmc_multilevel(state, B, r=2, b_cache=None, anneal=False,
-                    mcmc_equilibrate_args={}, anneal_args={}, shrink_args={},
-                    verbose=False):
-    r"""Equilibrate a MCMC from a starting state with a higher order, by performing
-    successive agglomerative initializations and equilibrations until the
-    desired order is reached, such that metastable states are avoided.
-
-    Parameters
-    ----------
-    state : Any state class (e.g. :class:`~graph_tool.inference.blockmodel.BlockState`)
-        Initial state. This state will **not** be modified during the algorithm.
-    B : ``int``
-        Desired model order (i.e. number of groups).
-    r : ``int`` (optional, default: ``2``)
-        Greediness of agglomeration. At each iteration, the state order will be
-        reduced by a factor ``r``.
-    b_cache : ``dict`` (optional, default: ``None``)
-        If specified, this should be a dictionary with key-value pairs of the
-        form ``(B, state)`` that contain pre-computed states of the specified
-        order. This dictionary will be modified during the algorithm.
-    anneal : ``bool`` (optional, default: ``False``)
-        If ``True``, the equilibration steps will use simulated annealing, by
-        calling :func:`~graph_tool.inference.mcmc.mcmc_anneal`, instead of
-        :func:`~graph_tool.inference.mcmc.mcmc_equilibrate`.
-    mcmc_equilibrate_args : ``dict`` (optional, default: ``{}``)
-        Arguments to be passed to :func:`~graph_tool.inference.mcmc.mcmc_equilibrate`.
-    mcmc_anneal_args : ``dict`` (optional, default: ``{}``)
-        Arguments to be passed to :func:`~graph_tool.inference.mcmc.mcmc_anneal`.
-    shrink_args : ``dict`` (optional, default: ``{}``)
-        Arguments to be passed to ``state.shrink``
-        (e.g. :meth:`graph_tool.inference.blockmodel.BlockState.shrink`).
-    verbose : ``bool`` or ``tuple`` (optional, default: ``False``)
-        If ``True``, progress information will be shown. Optionally, this
-        accepts arguments of the type ``tuple`` of the form ``(level, prefix)``
-        where ``level`` is a positive integer that specifies the level of
-        detail, and ``prefix`` is a string that is prepended to the all output
-        messages.
-
-    Notes
-    -----
-
-    This algorithm alternates between equilibrating the MCMC state and reducing
-    the state order (via calls to ``state.shrink``,
-    e.g. :meth:`graph_tool.inference.blockmodel.BlockState.shrink`).
-
-    This greatly reduces the changes of getting trapped in metastable states if
-    the starting point if far away from equilibrium, as discussed in
-    [peixoto-efficient-2014]_.
-
-    Returns
-    -------
-
-    state : The same type as parameter ``state``
-        This is the final state after the MCMC run.
-
-    References
-    ----------
-
-    .. [peixoto-efficient-2014] Tiago P. Peixoto, "Efficient Monte Carlo and
-       greedy heuristic for the inference of stochastic block models", Phys.
-       Rev. E 89, 012804 (2014), :doi:`10.1103/PhysRevE.89.012804`,
-       :arxiv:`1310.4378`
-    """
-
-    if "mcmc_equilibrate_args" in anneal_args:
-        raise ValueError("'mcmc_equilibrate_args' should be passed directly " +
-                         "to mcmc_multilevel(), not via 'anneal_args'")
-
-    mcmc_equilibrate_args = dict(dict(multiflip=False), **mcmc_equilibrate_args)
-    mcmc_args = mcmc_equilibrate_args.get("mcmc_args", {})
-    if not mcmc_equilibrate_args.get("gibbs", False):
-        mcmc_args["d"] = 0
-    else:
-        mcmc_args["allow_new_group"] = False
-    mcmc_equilibrate_args["mcmc_args"] = mcmc_args
-
-    while state.get_nonempty_B() > B:
-        B_next = max(min(int(round(state.get_nonempty_B() / r)),
-                         state.get_nonempty_B() - 1), B)
-
-        if b_cache is not None and B_next in b_cache:
-            if check_verbose(verbose):
-                print(verbose_pad(verbose) +
-                      "shrinking %d -> %d (cached)" % (state.get_nonempty_B(),
-                                                       B_next))
-            state = b_cache[B_next][1]
-            continue
-
-        if check_verbose(verbose):
-            print(verbose_pad(verbose) +
-                  "shrinking %d -> %d" % (state.get_nonempty_B(), B_next))
-        state = state.shrink(B=B_next, **shrink_args)
-        if anneal:
-            mcmc_anneal(state,
-                        **dict(anneal_args,
-                               mcmc_equilibrate_args=mcmc_equilibrate_args,
-                               verbose=verbose_push(verbose,
-                                                    "B=%d  " % state.get_nonempty_B())))
-        else:
-            mcmc_equilibrate(state,
-                             **dict(mcmc_equilibrate_args,
-                                    verbose=verbose_push(verbose,
-                                                         ("B=%d  " %
-                                                          state.get_nonempty_B()))))
-        if b_cache is not None:
-            mcmc_args = mcmc_equilibrate_args.get("mcmc_args", {})
-            entropy_args = mcmc_args.get("entropy_args", {})
-            b_cache[B_next] = (state.entropy(**entropy_args), state)
-    return state
-
-
 class MulticanonicalState(object):
     r"""The density of states of a multicanonical Monte Carlo algorithm. It is used
     by :func:`graph_tool.inference.mcmc.multicanonical_equilibrate`.
@@ -569,7 +457,7 @@ def multicanonical_equilibrate(m_state, f_range=(1., 1e-6), r=2,
         hf = m_state.get_flatness(allow_gaps=allow_gaps)
 
         if callback is not None:
-            callback(state, m_state)
+            callback(m_state)
 
         if check_verbose(verbose):
             print(verbose_pad(verbose) +
