@@ -19,7 +19,8 @@
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 from .. import Graph, _get_rng, Vector_size_t
-from . blockmodel import DictState, _bm_test
+from . blockmodel import DictState
+from . base_states import *
 from . partition_modes import contingency_graph
 
 from .. dl_import import dl_import
@@ -29,7 +30,7 @@ import numpy as np
 import math
 from scipy.special import gammaln
 
-class PartitionCentroidState(object):
+class PartitionCentroidState(MCMCState, MultiflipMCMCState, MultilevelMCMCState):
     r"""Obtain the center of a set of partitions, according to the variation of
     information metric or reduced mutual information.
 
@@ -97,101 +98,41 @@ class PartitionCentroidState(object):
         w = np.array(np.bincount(self.b), dtype="double")
         w = w[w>0]
         w /= w.sum()
-        return numpy.exp(-(w*log(w)).sum())
+        return np.exp(-(w*log(w)).sum())
 
+    @copy_state_wrap
     def entropy(self):
         return self._state.entropy()
 
-    def mcmc_sweep(self, beta=1.,d=.01, niter=1, allow_vacate=True,
-                   sequential=True, deterministic=False, verbose=False,
-                   **kwargs):
-        r"""Perform sweeps of a Metropolis-Hastings rejection sampling MCMC to sample
-        network partitions. See
-        :meth:`graph_tool.inference.blockmodel.BlockState.mcmc_sweep` for the
-        parameter documentation. """
-        oentropy_args = "."
-        mcmc_state = DictState(locals())
-        mcmc_state.vlist = Vector_size_t()
-        mcmc_state.vlist.resize(len(self.b))
-        mcmc_state.vlist.a = np.arange(len(self.b))
-        mcmc_state.state = self._state
-        mcmc_state.c = 0
-        mcmc_state.E = 0
-
-        test = kwargs.pop("test", True)
-        if _bm_test() and test:
-            Si = self.entropy()
+    def _get_entropy_args(self, kwargs):
+        return ""
 
+    def _mcmc_sweep_dispatch(self, mcmc_state):
         if self.RMI:
-            dS, nattempts, nmoves = \
-                libinference.rmi_mcmc_sweep(mcmc_state, self._state,
-                                            _get_rng())
+            return libinference.rmi_mcmc_sweep(mcmc_state, self._state,
+                                               _get_rng())
         else:
-            dS, nattempts, nmoves = \
-                libinference.vi_mcmc_sweep(mcmc_state, self._state,
-                                           _get_rng())
-
-        if _bm_test() and test:
-            Sf = self.entropy()
-            assert math.isclose(dS, (Sf - Si), abs_tol=1e-8), \
-                "inconsistent entropy delta %g (%g): %s" % (dS, Sf - Si)
-
-        if len(kwargs) > 0:
-            raise ValueError("unrecognized keyword arguments: " +
-                             str(list(kwargs.keys())))
-        return dS, nattempts, nmoves
-
-
-    def multiflip_mcmc_sweep(self, beta=1., psingle=None, psplit=1, pmerge=1,
-                             pmergesplit=1, d=0.01, gibbs_sweeps=10, niter=1,
-                             accept_stats=None, verbose=False, **kwargs):
-        r"""Perform sweeps of a merge-split Metropolis-Hastings rejection sampling MCMC
-        to sample network partitions. See
-        :meth:`graph_tool.inference.blockmodel.BlockState.mcmc_sweep` for the
-        parameter documentation."""
-        if psingle is None:
-            psingle = len(self.b)
-        gibbs_sweeps = max(gibbs_sweeps, 1)
-        nproposal = Vector_size_t(4)
-        nacceptance = Vector_size_t(4)
-        force_move = kwargs.pop("force_move", False)
-        oentropy_args = "."
-        mcmc_state = DictState(locals())
-        mcmc_state.state = self._state
-        mcmc_state.c = 0
-        mcmc_state.E = 0
-
-        test = kwargs.pop("test", True)
-        if _bm_test() and test:
-            Si = self.entropy()
+            return libinference.vi_mcmc_sweep(mcmc_state, self._state,
+                                              _get_rng())
 
+    def _multiflip_mcmc_sweep_dispatch(self, mcmc_state):
         if self.RMI:
-            dS, nattempts, nmoves = \
-                libinference.rmi_multiflip_mcmc_sweep(mcmc_state, self._state,
-                                                      _get_rng())
+            return libinference.rmi_multiflip_mcmc_sweep(mcmc_state,
+                                                         self._state,
+                                                         _get_rng())
         else:
-            dS, nattempts, nmoves = \
-                libinference.vi_multiflip_mcmc_sweep(mcmc_state, self._state,
-                                                     _get_rng())
+            return libinference.vi_multiflip_mcmc_sweep(mcmc_state, self._state,
+                                                        _get_rng())
 
-        if _bm_test() and test:
-            Sf = self.entropy()
-            assert math.isclose(dS, (Sf - Si), abs_tol=1e-8), \
-                "inconsistent entropy delta %g (%g): %s" % (dS, Sf - Si)
-
-        if len(kwargs) > 0:
-            raise ValueError("unrecognized keyword arguments: " +
-                             str(list(kwargs.keys())))
-
-        if accept_stats is not None:
-            for key in ["proposal", "acceptance"]:
-                if key not in accept_stats:
-                    accept_stats[key] = numpy.zeros(len(nproposal),
-                                                    dtype="uint64")
-            accept_stats["proposal"] += nproposal.a
-            accept_stats["acceptance"] += nacceptance.a
-
-        return dS, nattempts, nmoves
+    def _multilevel_mcmc_sweep_dispatch(self, mcmc_state):
+        if self.RMI:
+            return libinference.rmi_multilevel_mcmc_sweep(mcmc_state,
+                                                          self._state,
+                                                          _get_rng())
+        else:
+            return libinference.vi_multilevel_mcmc_sweep(mcmc_state,
+                                                         self._state,
+                                                         _get_rng())
 
 
 def variation_information(x, y, norm=False):
@@ -328,9 +269,8 @@ def mutual_information(x, y, norm=False):
     MI /= N
     MI += np.log(N)
     if norm:
-        N = mrs.sum()
-        Hx = -((nr * np.log(nr))[part == 0]).sum() + N * np.log(N)
-        Hy = -((nr * np.log(nr))[part == 1]).sum() + N * np.log(N)
+        Hx = -((nr * np.log(nr))[part == 0]).sum() / N + np.log(N)
+        Hy = -((nr * np.log(nr))[part == 1]).sum() / N + np.log(N)
         MI /= (Hx + Hy)/2
     return float(MI)
 
@@ -344,7 +284,8 @@ def reduced_mutual_information(x, y, norm=False):
     y : iterable of ``int`` values
         Second partition.
     norm : (optional, default: ``True``)
-        If ``True``, the result will be normalized in the range :math:`[0,1]`.
+        If ``True``, the result will be normalized such that the maximum
+        possible value is one.
 
     Returns
     -------
@@ -354,7 +295,7 @@ def reduced_mutual_information(x, y, norm=False):
     Notes
     -----
 
-    The reduced mutual information [newman_improved_2020]_ is defined as
+    The reduced mutual information (RMI) [newman_improved_2020]_ is defined as
 
     .. math::
 
@@ -380,6 +321,8 @@ def reduced_mutual_information(x, y, norm=False):
 
     which can take a maximum value of one.
 
+    Note that both the normalized and unormalized RMI values can be negative.
+
     This algorithm runs in time :math:`O(N)` where :math:`N` is
     the length of :math:`\boldsymbol x` and :math:`\boldsymbol y`.
 
